package laboratoire4.game;

import laboratoire4.IPawn;
import laboratoire4.Player;
import laboratoire4.pawns.PawnMovement;
import laboratoire4.pawns.SimulatedPawn;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Stack;
import java.util.function.Predicate;

public class SimulatedGame implements Game {
    private final SimulatedPawn[][] board;
    private final Player player;

    private final Stack<SimulatedPawn> removedPawns = new Stack<>();
    private final Stack<SimulatedPawn> previousMovedPawns = new Stack<>();

    public SimulatedGame(IPawn[][] board, Player player) {
        this.board = asMovingPawns(board);
        this.player = player;
    }

    public void move(IPawn pawn, PawnMovement movement) {
        int toRow = pawn.getRow() + pawn.getDirection();
        int toCol = pawn.getCol() + movement.getMove();
        SimulatedPawn capturedPawn = board[toRow][toCol];

        previousMovedPawns.push((SimulatedPawn) pawn);
        removedPawns.push(capturedPawn);

        board[pawn.getRow()][pawn.getCol()] = null;
        board[toRow][toCol] = (SimulatedPawn) pawn;
        pawn.move(movement);
    }

    public void revertMove() {
        SimulatedPawn pawn = previousMovedPawns.pop();
        SimulatedPawn capturedPawn = removedPawns.pop();

        board[pawn.getRow()][pawn.getCol()] = capturedPawn;

        pawn.revertMove();
        board[pawn.getRow()][pawn.getCol()] = pawn;
    }

    public SimulatedPawn getNextPawn(IPawn pawn, PawnMovement move) {
        int nextRow = pawn.getRow() + pawn.getDirection();
        int nextCol = pawn.getCol() + move.getMove();

        if (nextCol < 0 || nextCol > 7) {
            return null;
        }

        return board[nextRow][nextCol];
    }

    public SimulatedPawn[][] getBoard() {
        return board;
    }

    public Player getPlayer() {
        return player;
    }

    public boolean hasCaptured() {
        return removedPawns.peek() != null;
    }


    private Collection<SimulatedPawn> getPawns(Predicate<SimulatedPawn> predicate) {
        List<SimulatedPawn> pawns = new ArrayList<>();
        for (SimulatedPawn[] row : board) {
            for (SimulatedPawn pawn : row) {
                if (pawn != null && predicate.test(pawn)) {
                    pawns.add(pawn);
                }
            }
        }
        return pawns;
    }

    public static SimulatedPawn[][] asMovingPawns(IPawn[][] board) {
        SimulatedPawn[][] to = new SimulatedPawn[board.length][board.length];
        for (int row = 0; row < board.length; row++) {
            for (int col = 0; col < board.length; col++) {
                if (board[row][col] == null) {
                    continue;
                }

                SimulatedPawn pawn = SimulatedPawn.from(board[row][col]);
                to[row][col] = pawn;
            }
        }
        return to;
    }
}
