package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnMovement;
import laboratoire4.pawns.PawnUtils;
import laboratoire4.pawns.SimulatedPawn;

import java.util.Collection;
import java.util.stream.Collectors;

public class AttackStrategy extends MiniMaxStrategy {
    private static final int ATTACK_DISTANCE_FROM_GOAL = 4;

    private long originalMinPawnCount;
    private long originalMaxPusherCount;

    @Override
    public EvaluationResult getNextMove(Game game) {
        originalMinPawnCount = GameUtils.getMinPawnsAsStream(game.getBoard(), game.getPlayer()).count();
        originalMaxPusherCount = GameUtils.getMaxPawnsAsStream(game.getBoard(), game.getPlayer())
                .filter(IPawn::isPusher)
                .count();

        return super.getNextMove(game);
    }

    @Override
    protected int evaluateSimulation() {
        int score = 0;

        if (getMinPawns().stream().anyMatch(p -> p.getRow() == p.getPlayer().getGoal())) {
            return Integer.MIN_VALUE;
        }

        long pusherCount = getMaxPawns().stream().filter(IPawn::isPusher).count();
        if (pusherCount == 0) {
            return Integer.MIN_VALUE;
        }
        score -= Math.pow(originalMaxPusherCount - pusherCount, 2);

        for (SimulatedPawn pawn : getMaxPawns()) {
            if (pawn.getRow() == pawn.getPlayer().getGoal()) {
                return Integer.MAX_VALUE;
            }

            if (PawnUtils.hasEasyWinPath(game.getBoard(), pawn)) {
                return Integer.MAX_VALUE / 2;
            }

//            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
//            if (distanceFromGoal <= ATTACK_DISTANCE_FROM_GOAL) {
//                score += ATTACK_DISTANCE_FROM_GOAL - distanceFromGoal + 1;
//            }

            if (pawn.isPusher() && PawnUtils.canBeCaptured(game, pawn)) {
                score -= 10;  // On ne veut pas nécessairement bouger où on peut être capturé facilement
            }
        }

        int minPawnCount = getMinPawns().size();
        long capturedPawnCount = originalMinPawnCount - minPawnCount;
        score += Math.pow(capturedPawnCount, 2);

        return score;
    }

    @Override
    public int getWeight(Game game) {
        Collection<IPawn> maxPawns = GameUtils.getMaxPawns(game.getBoard(), game.getPlayer());

        int maxWeight = 0;
        for (IPawn pawn : maxPawns) {
            int weight = 0;

            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
            if (distanceFromGoal <= 1) {
                return WEIGHT_MAX;
            }

            if (distanceFromGoal == 2) {
                if (PawnUtils.hasEasyWinPath(game.getBoard(), pawn)) {
                    weight = 8;
                } else {
                    weight = 5;
                }
            }

            if (distanceFromGoal > 2 && distanceFromGoal <= ATTACK_DISTANCE_FROM_GOAL) {
                weight = 2;
            }

            if (weight > maxWeight) {
                maxWeight = weight;
            }
        }

        return maxWeight;
    }

    private boolean cantBeCaptured(Action<IPawn> action) {
        IPawn pawn = action.getPawn();
        PawnMovement movement = action.getMovement();

        int row = pawn.getRow() + pawn.getDirection();
        int col = pawn.getCol() + movement.getMove();

        return !PawnUtils.canBeCaptured(game, row, col, pawn.getPlayer());
    }
}
