package laboratoire4.strategies;

import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnMovement;
import laboratoire4.pawns.PawnUtils;

public class ImmediateDefenseStrategy implements Strategy {
    private static final int IMM_DEFENSE_DISTANCE_FROM_HOME = 1;

    @Override
    public EvaluationResult getNextMove(Game game) {
        // On utilise pas la méthode utilitaire, car on veut bouger la ligne de défense si nécessaire
        for (IPawn pawn : GameUtils.getMaxPawns(game.getBoard(), game.getPlayer())) {
            if (PawnUtils.distanceFromHome(pawn) > IMM_DEFENSE_DISTANCE_FROM_HOME) {
                continue;
            }

            for (PawnMovement movement : PawnMovement.values()) {
                if (pawn.isMoveValid(game.getBoard(), movement)) {
                    int nextRow = pawn.getRow() + pawn.getDirection();
                    int nextCol = pawn.getCol() + movement.getMove();
                    IPawn nextPawn = game.getBoard()[nextRow][nextCol];

                    if (nextPawn == null || PawnUtils.areSamePlayers(pawn, nextPawn)) {
                        continue;
                    }

                    return new EvaluationResult(pawn.getRow(), pawn.getCol(), movement);
                }
            }
        }

        return null;
    }

    @Override
    public int getWeight(Game game) {
        for (IPawn pawn : GameUtils.getMaxPawns(game.getBoard(), game.getPlayer())) {
            if (PawnUtils.distanceFromHome(pawn) > IMM_DEFENSE_DISTANCE_FROM_HOME) {
                continue;
            }

            for (PawnMovement movement : PawnMovement.values()) {
                if (pawn.isMoveValid(game.getBoard(), movement)) {
                    int nextRow = pawn.getRow() + pawn.getDirection();
                    int nextCol = pawn.getCol() + movement.getMove();
                    IPawn nextPawn = game.getBoard()[nextRow][nextCol];

                    if (nextPawn == null || PawnUtils.areSamePlayers(pawn, nextPawn)) {
                        continue;
                    }

                    return WEIGHT_MAX;
                }
            }
        }

        return 0;
    }
}
